/*
  Rui Santos
  Complete project details at https://RandomNerdTutorials.com/esp32-cam-post-image-photo-server/
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files.
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
*/

#include <Arduino.h>
#include <WiFi.h>
#include <WiFiClientSecure.h>
#include "soc/soc.h"
#include "soc/rtc_cntl_reg.h"
#include "esp_camera.h"

const char* ssid = "WLAN++";
const char* password = "1Q33omwR";
//const char* ssid = "WLAN+";
//const char* password = "XPmeg8i2U9";

// REPLACE WITH YOUR DOMAIN NAME or Raspberry Pi IP ADDRESS
//String serverName = "haeusle.svhub.de";
String serverName = "haeusle.hinz.casa";

// The default serverPath to post images
String serverPath = "/upload";
const int serverPort = 443;
WiFiClientSecure client; // for https

// CAMERA_MODEL_AI_THINKER
#define PWDN_GPIO_NUM     32
#define RESET_GPIO_NUM    -1
#define XCLK_GPIO_NUM      0
#define SIOD_GPIO_NUM     26
#define SIOC_GPIO_NUM     27

#define Y9_GPIO_NUM       35
#define Y8_GPIO_NUM       34
#define Y7_GPIO_NUM       39
#define Y6_GPIO_NUM       36
#define Y5_GPIO_NUM       21
#define Y4_GPIO_NUM       19
#define Y3_GPIO_NUM       18
#define Y2_GPIO_NUM        5
#define VSYNC_GPIO_NUM    25
#define HREF_GPIO_NUM     23
#define PCLK_GPIO_NUM     22

/*
  Doorbell pins
  -> Digital In/Out ports (PWM)

              _____________________
              |    | |_-|-__|  |    |
              |    |___________|    |
              |    |___________|    |
              |    |           |    |
              |    | ESP32-CAM |    |
      GPIO-4  | o  |           |  o | GND
      GPIO-2  | o  |           |  o | GPIO-1 Tx -> Rx Programmer
      GPIO-14 | o  |___________|  o | GPIO-3 Rx -> Tx Programmer
      GPIO-15 | o  |           |  o | VCC 3.3/5V Output
      GPIO-13 | o  |           |  o | GND
      GPIO-12 | o  |           |  o | GPIO-0 -> to GND for programming
      GND     | o  |           |  o | GPIO-16
      5V      | o  |           |  o |
               _____________________

*/



#define LED_PIN        2 
#define RELAY_PIN      12
#define PIR_PIN        13
//#define XYZ_PIN      16
#define BTN_PIN        14 // Doorbell Button

const int ledInterval = 5000;       // time for the button led to be on after pushing
const int timerInterval = 1000;     // min. time between each HTTPS POST image
const int relayInterval = 2000;     // time to keep relay active
unsigned long previousMillis = 0;   // last time image was sent
unsigned long previousLedMillis = 0;// last time the button was pushed

void setup() {
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0); 
  Serial.begin(115200);

  connectWiFi();

  Serial.println("Camera config ...");
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = Y2_GPIO_NUM;
  config.pin_d1 = Y3_GPIO_NUM;
  config.pin_d2 = Y4_GPIO_NUM;
  config.pin_d3 = Y5_GPIO_NUM;
  config.pin_d4 = Y6_GPIO_NUM;
  config.pin_d5 = Y7_GPIO_NUM;
  config.pin_d6 = Y8_GPIO_NUM;
  config.pin_d7 = Y9_GPIO_NUM;
  config.pin_xclk = XCLK_GPIO_NUM;
  config.pin_pclk = PCLK_GPIO_NUM;
  config.pin_vsync = VSYNC_GPIO_NUM;
  config.pin_href = HREF_GPIO_NUM;
  config.pin_sscb_sda = SIOD_GPIO_NUM;
  config.pin_sscb_scl = SIOC_GPIO_NUM;
  config.pin_pwdn = PWDN_GPIO_NUM;
  config.pin_reset = RESET_GPIO_NUM;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;

  // init with high specs to pre-allocate larger buffers
  if(psramFound()){
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 5;  //0-63 lower number means higher quality
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_CIF;
    config.jpeg_quality = 12;  //0-63 lower number means higher quality
    config.fb_count = 1;
  }

  Serial.println("Pin init ...");
  //Doorbell Button
  pinMode(BTN_PIN, INPUT);
  //Motion Sensor (PIR)
  pinMode(PIR_PIN, INPUT);
  pinMode(RELAY_PIN, OUTPUT);
  pinMode(LED_PIN, OUTPUT);

  Serial.println("Carmera init ...");
  // camera init
  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed with error 0x%x", err);
    delay(1000);
    ESP.restart();
  }

  //sendPhoto();
  
  digitalWrite(RELAY_PIN, LOW);
  digitalWrite(LED_PIN, LOW);
}

void loop() {

  // if button wasn't pressed since timerInterval
  if (millis() - previousMillis >= timerInterval) {

    //button pressed now
    if (! digitalRead(BTN_PIN)) {

      Serial.println("Button pressed ...");
      digitalWrite(RELAY_PIN, HIGH);    // activate relay
      digitalWrite(LED_PIN, HIGH);      // activate button LED
      
      sendEvent("doorbell_button", 1);  // send signal to server
      sendPhoto("doorbell_button");     // make photo and send to server
      previousMillis = millis();        // reset timer
    }
  }

  // Turn off relay after relayInterval
  if (digitalRead(RELAY_PIN)) {
    if (millis() - previousMillis >= relayInterval) {
      digitalWrite(RELAY_PIN, LOW);
    }
  }

  // Turn off LED after ledInterval
  if (digitalRead(LED_PIN)) {
    if (millis() - previousMillis >= ledInterval) {
      digitalWrite(LED_PIN, LOW);
    }
  }
/*
  else if (digitalRead(PIR_PIN)) {
    Serial.println("Motion detected ...");
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis >= timerInterval) {
      sendEvent("doorbell_motion");
      //sendPhoto("doorbell_motion");
      previousMillis = currentMillis;
    }
  }
*/
  
}

void sendEvent(const String& event_name, int num){
  if (client.connect(serverName.c_str(), serverPort)) {
    Serial.println("sendEvent() - Connection successful!");
    
    client.println("GET https://" + serverName + "/event?event=" + event_name + " HTTP/1.0");
    client.println("Host: " + serverName);
    //client.println("Connection: close");
    client.println();
  } else {
    Serial.println("sendEvent() - Connection failed!");
    if (num <= 5){
      connectWiFi();
      sendEvent(event_name, num + 1);
    }
  }
  //client.stop();
}

String sendPhoto(const String& data_name) {
  String getAll;
  String getBody;

  camera_fb_t * fb = NULL;
  fb = esp_camera_fb_get();
  if(!fb) {
    Serial.println("Camera capture failed");
    delay(1000);
    ESP.restart();
  }
  
  Serial.println("Connecting to server: " + serverName);

  if (client.connect(serverName.c_str(), serverPort)) {
    Serial.println("Connection successful!");    
    String head = "--ESP32\r\nContent-Disposition: form-data; name=\"" + data_name + "\"; filename=\"esp32-cam.jpg\"\r\nContent-Type: image/jpeg\r\n\r\n";
    String tail = "\r\n--ESP32--\r\n";

    uint32_t imageLen = fb->len;
    uint32_t extraLen = head.length() + tail.length();
    uint32_t totalLen = imageLen + extraLen;
  
    client.println("POST " + serverPath + " HTTP/1.1");
    client.println("Host: " + serverName);
    client.println("Content-Length: " + String(totalLen));
    client.println("Content-Type: multipart/form-data; boundary=ESP32");
    client.println();
    client.print(head);
  
    uint8_t *fbBuf = fb->buf;
    size_t fbLen = fb->len;
    for (size_t n=0; n<fbLen; n=n+1024) {
      if (n+1024 < fbLen) {
        client.write(fbBuf, 1024);
        fbBuf += 1024;
      }
      else if (fbLen%1024>0) {
        size_t remainder = fbLen%1024;
        client.write(fbBuf, remainder);
      }
    }   
    client.print(tail);
    
    esp_camera_fb_return(fb);
    
    int timoutTimer = 10000;
    long startTimer = millis();
    boolean state = false;
    //Blynk.notify("Someone is at the door..");
    //Blynk.setProperty(V1, "urls", uploadsPath);
    
    while ((startTimer + timoutTimer) > millis()) {
      Serial.print(".");
      delay(100);      
      while (client.available()) {
        char c = client.read();
        if (c == '\n') {
          if (getAll.length()==0) { state=true; }
          getAll = "";
        }
        else if (c != '\r') { getAll += String(c); }
        if (state==true) { getBody += String(c); }
        startTimer = millis();
      }
      if (getBody.length()>0) { break; }
    }
    Serial.println();
    client.stop();
    Serial.println(getBody);
  }
  else {
    getBody = "Connection to " + serverName +  " failed.";
    Serial.println(getBody);
  }
  return getBody;
}

void connectWiFi() {
  WiFi.mode(WIFI_STA);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);  
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println();
  Serial.print("ESP32-CAM IP Address: ");
  Serial.println(WiFi.localIP());
}
